/*********************************************************************/
/* File:   myCollecting.cpp                                          */
/* Author: Johannes Maierhofer                                       */
/* Date:   12. Dezember. 2019                                        */
/*********************************************************************/

#include <ngstd.hpp>
#include <nginterface.h>
#include <comp.hpp>

#include <python_comp.hpp>

namespace ngsolve
{
    using namespace std;

    using namespace ngstd;
    using namespace ngla;
    using namespace ngfem;
    using namespace ngcomp;
    using namespace ngmg;
}

using namespace ngsolve;

namespace mycollect {
    std::vector <Vector<double>> MyCollect(shared_ptr <FESpace> fes, shared_ptr <LinearFormIntegrator> lfi) {
        cout << "We collect force element vector" << endl;

        auto ma = fes->GetMeshAccess();

        int ne = ma->GetNE();

        auto rvecfList = std::make_shared < std::vector < Vector < double >> > ();
        rvecfList->resize(ne);

        /*
          Parallel iteration over elements using element coloring
          This automatically splits the lh into partial localheaps and
          calls the given lambda function with the element and the splitted lh
          We can create flatmatrices and -vectors on the local heap without
          memory management efficiently - the local heap will be reseted by
          IterateElements automatically.
        */
        LocalHeap lh(100000);
        IterateElements(*fes, VOL, lh, [&](FESpace::Element el, LocalHeap &lh) {
            const ElementTransformation &eltrans = ma->GetTrafo(el, lh);
            const FiniteElement &fel = fes->GetFE(el, lh);
            auto dofs = el.GetDofs();
            FlatVector<double> elvec(dofs.Size(), lh);
            lfi->CalcElementVector(fel, eltrans, elvec, lh);
            rvecfList->at(el.Nr()) = Vector<double> (elvec);
            /*
            rvecfList->at(el.Nr()).AssignMemory(dofs.Size(), lh);
            rvecfList->at(el.Nr()) = elvec;
            */	
            py::print(el.Nr());
            py::print(elvec);
            //py::print(rvecfList->at(el.Nr()));
            py::print(*rvecfList);
        });

        return *rvecfList;
    }
}
